//
//  SkynetConnect.hpp
//  LibSkynet
//
//  C/C++ API for Connecting to Skynet Server
//
//  Created by Miha Nedok on 30/09/16.
//  Copyright © 2016 Mike. All rights reserved.
//




#ifndef SkynetConnect_hpp
#define SkynetConnect_hpp
#include "SkynetLib.h"
/**
 *  Export Everything to Library
 */
BEGIN_EXTERN_C()


/**
 *  data for ping.
 */
typedef struct node_data_s{
  char *channel;
  char *appId;
  char *nodename;
  char *connectUri;
  char *application_session_id;
  char *vpnIp;
  char *publicIp;
  char *publicUri;
  int node_type;
  int beat_interval;
}node_data_s;
typedef node_data_s* node_data_p;



node_data_p skynet_connect_new_node_data();

json_t *skynet_connect_nodeinfo_to_json(node_data_p data);

node_data_p skynet_connect_json_to_nodeinfo(json_t *j_node_info);

/**
 * Initialize component.
 * This Should be called once per application.
 */
void skynet_connect_init();

/**
 * Verifiy the json_t response from skynet callers.
 * @param  result the json_t response
 */
#define SKYNET_RESULT_FAILED(result) (result == NULL || !json_boolean_value(json_object_get(result,"success")))

/**
 * Send a command with payload for channel.
 * @param  message_name Name of the message.
 * @param  payload      Data to send
 * @param connectUri             Url for Skynet (ex: tcp://10.0.0.1:9990)
 * @param application_id
 * @param application_session_id
 * @return              If sucessfull returns json parsed response
 */
json_t *skynet_send_internal(const char *message_name,json_t *payload, const char *connectUri, const char *application_id,const char *application_session_id);

/**
 * Send a command with payload for channel.
 * @param  channel_name Name of the channel we wish to reach.
 * @param  message_name Name of the message.
 * @param  payload      Data to send
 * @return              If sucessfull returns json parsed response
 */
json_t* skynet_send_internal_channel(const char *channel_name,const char *message_name,json_t *payload,const char *connectUri,const char *application_id,const char *application_session_id);

/*
 */

/**
 * Perform login on Skynet.
 *
 *
 *  @return success of login
 */
gboolean skynet_connect_login_app(const char *connectUri,const char *skynet_username, const char *skynet_password,const char *application_id,char **application_session_id);




/**
 *  Helper for Registering a node to Skynet server Media channel.
 *  Uses SkynetConnect to register node.
 *
 *  @param nodename               name of node
 *  @param nodetype               type of node
 *  @param vpnIp                  ip vpn
 *  @param publicIp               ip public
 *  @param connectUri             connection url for Skynet
 *  @param applicationId          ApplicationId
 *  @param application_session_id Application's sessionId
 *
 *  @return success
 */
gboolean skynet_register_node(const char *channel,const char *nodename,int nodetype, const char *vpnIp, const char *publicIp, const char *connectUri,const char *applicationId, const char *application_session_id);

/**
 *  Helper for Sending a PING for a node to Skynet server Media channel.
 *
 *  @param nodename               name of node
 *  @param nodetype               type of node
 *  @param connectUri             connect url of Skynet
 *  @param applicationId          ApplicationId
 *  @param application_session_id Application's sessionId
 *
 *  @return success
 */
gboolean skynet_ping_from_node(const char *nodename,int nodetype, const char *connectUri,const char *applicationId, const char *application_session_id);

/**
 *  Same as skynet_ping_from_node but taking only one data parameter which embeds all needed info.
 *
 *  @param channel The registering channel to call.
 *  @param data Info
 *
 *  @return success
 */
gboolean skynet_ping_from_node_data(const char *channel,node_data_p data);


/**
 *
 *
 *  @param channel
 *  @param nodename
 *  @param nodetype
 *  @param connectUri
 *  @param appId
 *  @param application_session_id
 *  @param beatInterval
 *
 *  @return Success
 */
gboolean skynet_heartbeat_sender(const char *channel,const char *nodename,int nodetype,const char *connectUri, const char *appId,const char *application_session_id,int beatInterval);

END_EXTERN_C();

#endif /* SkynetConnect_hpp */
