/*
 * SkynetLib.h
 *
 * Author: Miha Nedok <mike@mike.si>
 */


#ifdef __cplusplus

extern "C" {
  // #include <cstddef>
#endif


#ifndef BEGIN_EXTERN_C


#ifdef __cplusplus

  // Skynet Return Codes
  enum SkynetRC : int {
    // Generic OK value
    OK																								= 0,
    // Database Error
    ERR_DB																						= 12,
    // Lock acquire or release error
    ERR_LOCK																					= 13,
    TRANS_ERR_NOT_VALID_PRODUCT												= 1,
    TRANS_ERR_INSUFFICIENT_FUNDS											= 2,
    TRANS_ERR_USER_INVALID														= 3,
    TRANS_ERR_USER_NOTACTIVE													= 4,
    TRANS_ERR_RECIPIENT_INVALID												= 5,
    TRANS_ERR_LOCKING																	= 6,
    TRANS_ERR_INTERNAL_ERR														= 7,
    TRANS_ERR_DATABASE_PROBLEM												= 8,
    TRANS_ERR_ACCOUNT_NOT_OK													= 9,
    TRANS_ERR_INIT_FAILED															= 10,
    // Passed parameters are not correct
    TRANS_ERR_PARAMETERS															= 11,
    // Error with Account
    UPDATE_BALANCE_ERR_ACCT_PROBLEM										= 14,
    ERR_GENERIC																				= 15,
    ERR_ARRAY_EMPTY																		= 16,
    ERR_JSON_DUMP																			= 17
  };



#define BEGIN_EXTERN_C() extern "C" {
#define END_EXTERN_C() }
#else
#define BEGIN_EXTERN_C()
#define END_EXTERN_C()
#endif



#endif



#include <glib.h>
#include <gmodule.h>
  // #include <stdint.h>
  //#include <inttypes.h>
  //#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
  // #include <unistd.h>
#include <sys/socket.h>
#include <sys/socket.h>
  // #include <netdb.h>
  //#include <netinet/in.h>
#include <fcntl.h>
#include <sys/un.h>
  //#include <netinet/ip.h>
  // #include <arpa/inet.h>
#include <netinet/tcp.h>        /* For TCP_NODELAY */
#include <sys/poll.h>
#include <sys/stat.h>
  //#include <assert.h>
#include <errno.h>
#include <locale.h>
#include <time.h>
#include <unistd.h>
  // #include <mysql.h>

#include <syslog.h>
#include <curl/curl.h>
#include <jansson.h>


#include <ldap.h>


#include <microhttpd.h>

#include <zdb.h>



#include <liboath/oath.h>


#include "SkynetLibUtils.h"
#include "SkynetTypes.h"
#include "SkynetHTTP.h"
#include "SkynetConnect.h"


#define BLOCK_OF_1K	1024UL
#define BLOCK_OF_8K 8*BLOCK_OF_1K



#ifndef SKYNETLIB_H_
#define SKYNETLIB_H_


#include "PhpClient.h"

#define SKYNET_EXTERN extern "C"



#ifdef __cplusplus


#endif


  void SkynetLibReadConfig(json_t *configRoot,ServiceConfig **config);
  void SkynetLibLoadConfig(const char *path,ServiceConfig **config);


  void SkynetLibServerCommandsPopulate(void* pCmdList,GHashTable *cmdHashTable);
  struct SkynetServerCommand* SkynetLibServerCommandLookup(const char *cmdName,GHashTable *cmdHashTable);
  //void genericPHPrelay(SkynetServerMessage *msg);


  gboolean SkynetLockFileCreate(const char *lockFile);
  void SkynetLockFileDelete(const char *lockFile);


  void SkynetLDAPAuth(const char *ldapServer,short port, const char *dn, const char *password, int *resultCode);

  /**
   *  Generic SQL Prepared Statement Execute.
   *  IF Execute fails then the error is logged with syslog.
   *
   *  @param con  Connnection
   *  @param prep The Prepared Statement to Exec.
   *
   *  @return Success
   */
  //   gboolean SkynetLibSqlPrepExecute(Connection_T con,PreparedStatement_T prep);

  /**
   * Push a message for global listener
   */
  void SkynetLibWsPushForUser(const char *userId,const char *payload);

  /**
   *  Tries to determine the machine hostname
   *
   *  @return Hostname or NULL
   */
  char* SkynetGetHostname();

  /**
   * Check char* param
   */
#define SK_CHECK_PARAM(param) param == NULL || !strlen(param)



#define JSON_RESULT_SET_ERR(jsonDest,err,code){\
json_object_set_new((json_t*)jsonDest,"errorMessage",json_string(err));\
json_object_set_new((json_t*)jsonDest,"errorCode",json_integer(code));\
syslog(LOG_ERR,"Function: [%s] Error: [%s][%d]",msg->messageName,err,code);\
}

#define FUNCTION_RESULT_SUCCESS msg->success = TRUE;
#define FUNCTION_RESULT_FAILED	msg->success = FALSE;
#define JSON_SET_INCORRECT_PARAMETERS msg->success = FALSE; JSON_RESULT_SET_ERR(msg->jsonResult,"Parameters incorrect",-1)
#define JSON_SET_INCORRECT_PARAMETERS_MSG(invParamErrMsg) msg->success = FALSE;JSON_RESULT_SET_ERR(msg->jsonResult,invParamErrMsg,-1)
#define SKYNET_DECLARE_SERVER_FUNCTION(functionName) void functionName(SkynetServerMessage *msg)
#define SKYNET_IMPLEMENT_SERVER_FUNCTION(functionName) SKYNET_EXTERN void functionName(SkynetServerMessage *msg)

#define SKYNET_DECLARE_JSON_FUNCTION(functionName) void functionName(const char *mesageName,json_t *j_request)
#define SKYNET_IMPLEMENT_JSON_FUNCTION(functionName) void functionName(const char *mesageName,json_t *j_request)

#define PREP_PAR_BEGIN(){\
int incPara = 0;\

#define PREP_PAR_END() }

#define PREP_PAR_ADD_STR(prep,string) PreparedStatement_setString(prep,++incPara,string)
#define PREP_PAR_ADD_INT(prep,integer) PreparedStatement_setInt(prep,++incPara,integer)
#define PREP_PAR_ADD_DOUBLE(prep,doubleval) PreparedStatement_setDouble(prep,++incPara,doubleval)

#define PAYLOAD_GET_STR(key) json_string_value(json_object_get((json_t*)msg->payload,key))
#define PAYLOAD_GET_INT(key) json_integer_value(json_object_get((json_t*)msg->payload,key))
#define PAYLOAD_GET_JSON(key) json_object_get((json_t*)msg->payload,key)

#define GETSQLCONNECTION(appId) Connection_T con = globalManager->applicationSql(appId);
#define GETSQLCONNECTIONMSG GETSQLCONNECTION(msg->appId)

/*
 #define GETMONGOCONNECTION syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(false);connection->connect(config->mongoHost)
 #define GETMONGOCONNECTIONGLOB syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(false);connection->connect(SkynetConfig->mongoHost)
 #define GETMONGOCONNECTION_IN_CMDS syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(false);connection->connect(msg->config->mongoHost)
 #define GETMONGOCONNECTION_IN_CMDS_GM_APPID(appId) syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(false);connection->connect(globalManager->applicationMongoHost(appId))
 #define GETMONGOCONNECTION_IN_CMDS_MODULE syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(false);connection->connect(config->mongoHost)
 #define GETMONGOCONNECTION_HOST(host) syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(false);connection->connect(host)
 #define GETMONGOCONNECTION_GMPTR_APPID(gmPtr,appId) syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(false);connection->connect(gmPtr->applicationMongoHost(appId))

*/


#define GETMONGOCONNECTION syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(true);connection->connect(config->mongoHost);connection->auth(BSON("username" << "user" << "password" << "pass"))
#define GETMONGOCONNECTIONGLOB syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(true);connection->connect(SkynetConfig->mongoHost)
#define GETMONGOCONNECTION_IN_CMDS syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(true);connection->connect(msg->config->mongoHost)
#define GETMONGOCONNECTION_IN_CMDS_GM_APPID(appId) syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(true);connection->connect(globalManager->applicationMongoHost(appId))
#define GETMONGOCONNECTION_IN_CMDS_MODULE syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(true);connection->connect(config->mongoHost)
#define GETMONGOCONNECTION_HOST(host) syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(true);connection->connect(host)
#define GETMONGOCONNECTION_GMPTR_APPID(gmPtr,appId) syslog(LOG_WARNING,"ACQCONN:[%s][%d]",__FILE__,__LINE__);DBClientConnection *connection = new DBClientConnection(true);connection->connect(gmPtr->applicationMongoHost(appId))
#define LOGMONGOASSERTEXCEPTION(e) syslog(LOG_ERR,"DB AssertionException [%s][%d] E:[%s]",__FILE__,__LINE__,e.getInfo().toString().c_str())
#define MONGOLOGEXCEPTION syslog(LOG_ERR,"MongoDB Exception: [%s][%d] [%s]",__FILE__,__LINE__,e.what())


#define COLLECTIONSTRING(collectionName) string(string(config->mongoDb) + "." + collectionName)
#define COLLECTIONSTRING_APPID(appId,collectionName) string(globalManager->applicationMongoDb(appId) + string(".")+string(collectionName))
  // GlobalManger internal macro
#define _COLLECTIONSTRING_APPID(appId,collectionName) string(this->applicationMongoDb(appId) + string(".")+string(collectionName))
#define _COLLECTIONSTRING_APPID_OBJ(obj,appId,collectionName) string(obj->applicationMongoDb(appId) + string(".")+string(collectionName))
#define COLLECTIONSTRING_MSG(collectionName) COLLECTIONSTRING_APPID(msg->appId,collectionName)
#define COLLECTIONSTRING_DBSETTING(collectionName) (dbSetting&&strlen(dbSetting)>0)?string(string(dbSetting)+"."+string(collectionName)):string(string(msg->config->mongoDb) + "." + collectionName)

#define FUNCTION_BAIL_EXCEPTION {\
delete connection;\
syslog(LOG_ERR,"Internal Error: [%s] [%s] where:[%s] at:[%d] t:[%ld]",e.what(),__FUNCTION__,__FILE__,__LINE__,time(NULL));\
msg->success = FALSE;\
bPayload.append("message","Internal Server Error");\
bPayload.append("code",500);\
END_BSON_RESULT;\
}

#define FUNCTION_BAIL_INSUFFICIENT_PRIVILEGES {msg->success = FALSE; JSON_RESULT_SET_ERR(msg->jsonResult,"Insufficient Privileges to run the function",-2);}

#define FUNCTION_LOG_EXCEPTION {\
syslog(LOG_ERR,"Internal Error: [%s] [%s] where:[%s] at:[%d] t:[%ld]",e.what(),__FUNCTION__,__FILE__,__LINE__,time(NULL));\
}

#define FUNCTION_BAIL_INVALIDPARAMS JSON_SET_INCORRECT_PARAMETERS
#define FUNCTION_BAIL_INVALIDPARAMS_MSG(invParamErrMsg) JSON_SET_INCORRECT_PARAMETERS_MSG(invParamErrMsg)


#define _BSON_ADD_STR(bson,key,val){ if (val){bson.append(key,val);} else {bson.append(key,"");} }

  /**
   * Start BSON result
   */
#define DISCARD_BSON_RESULT delete (BSONObjBuilder*)msg->bsonResult; msg->bsonResult = NULL;
#define BEGIN_BSON_RESULT if(!msg->bsonResult){	msg->bsonResult = (void*)new BSONObjBuilder();}\
BSONObjBuilder *bResult = (BSONObjBuilder*)msg->bsonResult;\
BSONObjBuilder bPayload;

  /**
   * Put payload object (bPayload) to bsonResult
   */
#define END_BSON_RESULT bResult->appendElements(bPayload.obj());


#define BEGIN_JSON_RESULT SkynetJsonBuilder jBuilder;
#define END_JSON_RESULT   json_object_update(msg->jsonResult,jBuilder.getJson());json_decref(jBuilder.getJson());


  /**
   * Server support for Grid Data for client
   */
#define GRID_BEGIN(query) BSONArrayBuilder bRowData
#define GRID_ADD_INFO(records,limit) bPayload.append("records",records); bPayload.append("limit",limit)
#define GRID_ADD_DATA(rowdata) bPayload.append("rows",bRowData.arr())


  /**
   * Bail out of function if anonymous user
   */
#define BAIL_IF_ANON 	if (sessionManager->isAnonUser(msg->sessionId)){\
msg->success = false;\
syslog(LOG_ERR,"Anon user tried to access: [%s] code:[403]",msg->messageName);\
BEGIN_BSON_RESULT;\
bPayload.append("errorMessage","Access denied!");\
bPayload.append("errorCode",403);\
END_BSON_RESULT;\
return;\
}


#define SK_WS_SOCKET "ipc:///tmp/skwswrk.sock"

  void skynet_error_set(skynet_error_t* serror,char *format,...);


  void* SkynetPasswordHash(const char *password);
  int  SkynetPasswordHashVerify(const char *epassword, const char *password);

  char* SkynetBin2Hex(char *bin,int blen);


  // End Security - Passwords


#ifndef EXPTR
#define EXPTR(ptr) (*ptr)
#endif


  /* Global Commands Utils */

  inline struct SkynetServerCommand* SkynetCommandLookup(const char *command,GHashTable *cmdHash){
    return (struct SkynetServerCommand*)g_hash_table_lookup(cmdHash,(gconstpointer)command);
  }

  inline SkynetServerMessage* SkynetCommandExec(const char *command, GHashTable *cmdHash){
    struct SkynetServerCommand* cmd = SkynetCommandLookup(command,cmdHash);
    if (!cmd){
      return NULL;
    }
    SkynetServerMessage *msg = skynet_server_message_alloc();
    msg->jsonResult = json_object();
    cmd->cmd(msg);
    return msg;
  }



#ifndef mpGFreePtr
#define mpGFreePtr(ptr){if(ptr!=NULL){g_free(ptr);}}
#endif

#endif /* SKYNETLIB_H_ */

#ifdef __cplusplus
}
#endif
